;
;   Module name      :	LogUtil.au3
;   Version          :	1.0
;   Started on       :	June 7, 2012
;   Completed on     :	June 11, 2012
;   Author           :  J.Hubb
;
;   Module type      :	Library
;   Description      :	Utilities for log handling
;
;   Developed under  :	AutoIt 3.2.10.0
;   Sw requirements  :	None
;
;   Parameters       :	None
;   Notes - usage    :	The library exports 2 functions for log handling:
;							LURenWithDate		Renames a log file using its current name and
;												appending its file date in the form YYYYMMDD
;							LURenWithDateTime	Same of LURenWithDate but append time too as hhmmss
;							LURotate			Rotates a log file, keeping a specified number
;												of copies; warning: not implemented yet!
;							LUTail				Keeps a specified number of rows of a log file,
;												creating a backup copy of the previous version
;


;--- Global directives -----------------------------------------------------------------------------



#include-once

Opt("MustDeclareVars", 1)



;--- Include files ---------------------------------------------------------------------------------



#include <file.au3>



;--- Public constants ------------------------------------------------------------------------------



const $LU_FILE_NAME = "LogUtil.au3"
const $LU_MODULE_VER = "1.0"
const $LU_MODULE_NAME = $LU_FILE_NAME & " " & $LU_MODULE_VER & "; Open Source library on June 11, 2012 by J.Hubb."

; Error codes
const $LU_NO_ERROR = 0		; No error
const $LU_REN_ERR = 1		; Rename error
const $LU_SRC_ERR = 2		; Source file cannot be read
const $LU_DST_ERR = 3		; Destination file cannot be written



;--- Public variables ------------------------------------------------------------------------------



; String separator inserted between Log file name and date for function LURenWithDate()
global $LU_date_sep = "-"

; String separator inserted between date and time for function LURenWithDateTime()
global $LU_time_sep = "-"

; Extension of backup files for function LUTail()
global $LU_bak_ext = ".bak"



;--- Private constants -----------------------------------------------------------------------------



;--- Private variables -----------------------------------------------------------------------------



;--- Public functions code -------------------------------------------------------------------------



func LURotate($log_path, $num = 10)
;---
; Rotates the log file $log_path, keeping the specified number $num of copies.
; Warning: not implemented yet!
;
; $log_path		path of log file
; $num			number of copies to keep of previous log files
; ret			1 on success, 0 on failure
;---
	local $n
	
	; The following instruction is needed just to avoid warnings on not used variables.
	if (False) then $n = $log_path == $num == $n
	return 0
endfunc



func LURenWithDate($log_path)
;---
; Renames the log file $log_path, in the same directory, using the same extension and a name build
; as "<CurrentName>-YYYYMMDD". Separator can be changed using related global variable.
;
; $log_path		path of log file
; ret			1 on success, 0 on failure
;---
	local $drive, $dir, $name, $ext, $datetime
		
	_PathSplit($log_path, $drive, $dir, $name, $ext)
	$datetime = FileGetTime($log_path, -1, 1)
	return FileMove($log_path, $drive & $dir & $name & $LU_date_sep & StringLeft($datetime, 8) & $ext, 1)
endfunc



func LURenWithDateTime($log_path)
;---
; Renames the log file $log_path, in the same directory, using the same extension and a name build
; as "<CurrentName>-YYYYMMDD-hhmmss". Separators can be changed using related global variables.
;
; $log_path		path of log file
; ret			1 on success, 0 on failure
;---
	local $drive, $dir, $name, $ext, $datetime
		
	_PathSplit($log_path, $drive, $dir, $name, $ext)
	$datetime = FileGetTime($log_path, -1, 1)
	return FileMove($log_path, $drive & $dir & $name & $LU_date_sep & StringLeft($datetime, 8) & _
		$LU_time_sep & StringRight($datetime, 6) & $ext, 1)
endfunc



func LUTail($log_path, $rows = 100)
;---
; Keeps the specified number of rows $rows from the tail of the log file $log_path anche creates a copy
; of the original log file.
;
; $log_path		path of log file
; $rows			number of tailing rows to be kept
; ret			1 on success, 0 on failure setting @error
;---
	local $drive, $dir, $name, $ext
	local $bak_path, $tot_lines, $src, $dst, $line
		
	_PathSplit($log_path, $drive, $dir, $name, $ext)
	$bak_path = $drive & $dir & $name & $LU_bak_ext
	if (not FileMove($log_path, $bak_path, 1)) then
		SetError($LU_REN_ERR)
		return 0
	endif
	$tot_lines = _FileCountLines($bak_path);
	if ($tot_lines <= $rows) then			; Speed-up the full copy and exit
		FileCopy($bak_path, $log_path, 1)
		return 0
	endif
	$src = FileOpen($bak_path, 0)
	if ($src = -1) then
		SetError($LU_SRC_ERR)
		return 0
	endif
	$dst = FileOpen($log_path, 2)
	if ($src = -1) then
		SetError($LU_DST_ERR)
		return 0
	endif
	FileReadLine($src, $tot_lines - $rows)
	while True
		$line = FileReadLine($src);
		if (@error = -1) then
			ExitLoop
		endif
		if (FileWriteLine($dst, $line) < 0) then
			SetError($LU_DST_ERR)
			return 0
		endif
	wend
	FileClose($src)
	FileClose($dst)
	SetError($LU_NO_ERROR)
	return 1
endfunc



;--- Private functions code ------------------------------------------------------------------------



;--- Initialization code ---------------------------------------------------------------------------


